package com.sample;

import java.util.HashMap;
import java.util.Map;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.StatefulKnowledgeSession;

/**
 * This is a sample class to launch a rule.
 */
public class DroolsTest {

	public static final void main(String[] args) {
		try {
			// load up the knowledge base
			KnowledgeBase kbase = readKnowledgeBase();
			StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
			KnowledgeRuntimeLogger logger = KnowledgeRuntimeLoggerFactory.newFileLogger(ksession, "test");
			// go !
			
			Map<String, Object> dataMap = new HashMap<String, Object>();
			dataMap.put("c0", "foo");
			dataMap.put("c1", "bar");
			dataMap.put("c3", "bas");
			ksession.insert(new Data(0, dataMap));

			Map<String, Object> queryMap = new HashMap<String, Object>();
			queryMap.put("c0", "foo");
			queryMap.put("c1", "bar");
			//queryMap.put("c3", "bas!");
			ksession.insert(new Query(0, queryMap));
			
			ksession.fireAllRules();
			logger.close();
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	private static KnowledgeBase readKnowledgeBase() throws Exception {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("Sample.drl"), ResourceType.DRL);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
		if (errors.size() > 0) {
			for (KnowledgeBuilderError error: errors) {
				System.err.println(error);
			}
			throw new IllegalArgumentException("Could not parse knowledge.");
		}
		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		return kbase;
	}

	public static class Data {
		
		private final int id;
		private final Map<String, Object> values = new HashMap<String, Object>();
		
		public Data(int id, Map<String, Object> values) {
			this.id = id;
			this.values.putAll(values);
		}

		public int getId() {
			return id;
		}
		
		public Map<String, Object> getValues() {
			return values;
		}
	}

	public static class Query {
		
		private final int id;
		private final Map<String, Object> criteria = new HashMap<String, Object>();
		
		public Query(int id, Map<String, Object> criteria) {
			this.id = id;
			this.criteria.putAll(criteria);
		}

		public int getId() {
			return id;
		}
		
		public boolean dataMatch(Data data) {
			System.out.println("Comparing " + data.getValues() + " and " + criteria);
			if(data.getValues().keySet().containsAll(criteria.keySet())) {
				for(String key : criteria.keySet()) {
					Object critValue = criteria.get(key);
					Object dataValue = data.getValues().get(key);
					if(critValue == null && dataValue != null) {
						return false;
					} else if(critValue != null && dataValue == null) {
						return false;
					} else if(!critValue.equals(dataValue)) {
						return false;
					}
				}
				return true;
			} else {
				return false;
			}
		}
	}
}